#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Desc:
# vebzeitl is a program that builds webpages from some simple rules and
# does some action to some few tags.

import sys, os, re, time
import ConfigParser
import shutil

class LinkReader:
    reader = None
    linkfile = str()
    
    def __init__( self, linkfile ):
        # do magic
        print "Got %s as linkfile" % linkfile
        self.linkfile = linkfile
        self.reader = ConfigParser.ConfigParser()
        self.reader.read( linkfile )

    def check_sect( self, target ):
        """Makes the program die if the section doesn't exist"""
        if not self.reader.has_section( target ):
            print " " * 5 + "Error: no link %s in linkfile" % target
            sys.exit( 1 )
        
    def get_page( self, target ):
        # do magic
        self.check_sect( target )

        if not self.reader.has_option( target, "page" ):
            print " " * 5 + "Error: missing page for %s in %s" %(
                target, self.linkfile )
            exit( 1 )
            
        return self.reader.get( target, "page" )

    def get_text( self, target ):
        # do magic
        self.check_sect( target )

        if not self.reader.has_option( target, "text" ):
            print " " * 5 + "Error: missing text for %s in %s" %(
                target, self.linkfile )
            exit( 1 )
        
        return self.reader.get( target, "text" )

class Main_Program:
    # variables
    input_template = str()
    input_suffix = str()
    input_ignore = list()
    input_root = str()
    input_linkreader = None
    output_suffix = str()
    output_root = str()
    
    menu_files_list = list()
    template_files_list = list()
    input_files_list = list()
    
    def parse_settings( self, conf_file ):
        """Parse the setting file and put the variables where they belong!"""
        def get_opt( confp, sect, opt ):
            """Checks so a option in a section exists, and returns the value, else exits"""
            if confp.has_option( sect, opt ):
                tmpconf = confp.get( sect, opt )
                return tmpconf
            else:
                print "Error: missing option %s in section %s" %( opt, sect )
                sys.exit( 1 )
            
        def check_sect( confp, sect ):
            """Exits if the section is not existing else nothing"""
            if not confp.has_section( sect ):
                print "Error: missing section %s" %( sect )
                sys.exit( 1 )
            
        print "Config file: %s" % conf_file
        if conf_file.count('/') > 0:
            conf_dir = conf_file[:conf_file.rfind('/') +1]
            conf_file = conf_file[conf_file.rfind('/') +1:]
            print "entering: %s" % conf_dir
            os.chdir( conf_dir )
        print "Parsing: %s" % conf_file
        confp = ConfigParser.ConfigParser()
        confp.read( conf_file )
        check_sect( confp, "input" )
        check_sect( confp, "output" )
        self.input_root = get_opt( confp, "input", "root" )
        self.input_template = get_opt( confp, "input", "template" )
        self.input_menu = get_opt( confp, "input", "menu" )
        self.input_suffix = get_opt( confp, "input", "suffix" )
        self.input_ignore = get_opt( confp, "input", "ignore" ).split(' ')
        self.input_time_format = get_opt( confp, "input", "time_format" )
        
        self.input_linkreader = LinkReader( get_opt( confp,
                                                     "input",
                                                     "links" ) )
        self.output_root = get_opt( confp, "output", "root" )
        self.output_suffix = get_opt( confp, "output", "suffix" )
        print "Parsing: done"
        
    def scan_input( self ):
        """Find inputs, templates etc"""
        print "scanning: '%s'" % self.input_root
        for root, dirs, files in os.walk( self.input_root ):
            for file_name in files:
                path = root
                if path[-1] != '/':
                    path += '/'
                # test how we should process this
                if self.is_ignored( file_name ):
                    print "Ignored: %s" % path + file_name
                elif file_name == self.input_template:
                    print "Found template: %s" %( path + file_name )
                    self.template_files_list.append( [ path, file_name ] )
                elif file_name == self.input_menu:
                    print "Found menu: %s" %( path + file_name )
                    self.menu_files_list.append( [ path, file_name ] )
                elif file_name.find( self.input_suffix ) != -1:
                    targ_path = path.replace( self.input_root, self.output_root )
                    if not os.path.exists( targ_path ):
                        os.mkdir( targ_path )
                    print "Found input: %s" %( path + file_name )
                    self.input_files_list.append( [ path, file_name ] )
                else:
                    # its a unknown file type, just copy it
                    self.copy_input( path, file_name )
        print "Finnished scanning of: '%s'" % self.input_root
        for x in self.input_files_list:
            self.compile_input( x[0], x[1] )
        print "All done"
        
    def copy_input( self, path, file_name ):
        """copy's the input file, ig this is a image or something that should
        not be compiled"""
        targ_path = path.replace( self.input_root, self.output_root )
        print "Copying: %s to %s" %( path + file_name, targ_path )
        if not os.path.exists( targ_path ):
            os.mkdir( targ_path )
        shutil.copy( path + file_name, targ_path )

    def compile_input( self, path, file_name ):
        """compiles ye file and writes to zi output"""
        def list_to_str( listig ):
            string = str()
            for x in listig:
                x = x.strip( '\n' )
                string += x + '\n'
            return string
        
        print "Compiling: %s" % path + file_name
        input_text = list_to_str( open( path + file_name, 'r' ))
        template, menu = self.get_template_and_menu( path )
        print " * Template: %s" % template
        template_text = list_to_str( open( template, 'r' ).readlines())
        print " * Menu: %s" % menu
        menu_text = list_to_str( open( menu, 'r' ).readlines())
        targ_path = path.replace( self.input_root, self.output_root )
        
        # parser
        # if we wish to allow comments etc we must rewrite this
        template_text = template_text.replace( '<veb menu/>', menu_text )
        template_text = template_text.replace( '<veb page/>', input_text )
        # path
        if template_text.count( '<veb path/>' ) != 0:
            if targ_path != self.output_root:
                template_text = template_text.replace( '<veb path/>', targ_path.replace( self.output_root[:-1], "" ))
            else:
                template_text = template_text.replace( '<veb path/>', "" )
        # path_links
        if template_text.count( '<veb path_links' ) != 0:
            start = template_text.find( '<veb path_links "' )
            end = template_text[start:].find( '"/>' ) + start
            name = template_text[start:end].replace( '<veb path_links "', "" )
            line = targ_path.replace( self.output_root[:-1], name )
            if line.count ('/' ) == 1:
                line = line.strip( '/' )
            else:
                # make links!
                lines = line.split( '/' )[:-1]
                level = len( lines ) -1
                line = ""
                for x in lines:
                    if level != 0:
                        line += '<a href="%s">%s</a>/' %( '../' * level, x )
                    else:
                        line += x
                    level -= 1
            template_text = template_text.replace( template_text[start:end +3], line )
        # root_dots
        if template_text.count( '<veb root_dots/>' ) != 0:
            if targ_path != self.output_root:
                line = targ_path.replace( self.output_root[:-1], "" )
                nline = str()
                for x in range( 1, line.count( '/' )):
                    nline += "../"
                template_text = template_text.replace( '<veb root_dots/>',
                                                       nline )
            else:
                template_text = template_text.replace( '<veb root_dots/>', "" )
        # Last edited stamp, usefull for leting readers know when the
        # last time a page was updated was.
        if template_text.count( '<veb last_edit/>' ) != 0:
            ctime = os.stat( path + file_name ).st_ctime
            template_text = template_text.replace(
                '<veb last_edit/>', time.strftime( self.input_time_format,
                                                   time.localtime( ctime ) ) )
        
        # links
        print " * Looking for links"
        while re.search( '<veb link "(\S+)"/>', template_text ) != None:
            # find target name
            target = re.search( '<veb link "(\S+)"/>', template_text ).group(1)
            print " " * 4 + " + '%s'" % target
            
            # insert link
            template_text = re.sub( '<veb link "(\S+)"/>',
                                    '<a href="%s">%s</a>' %(
                self.input_linkreader.get_page( target ),
                self.input_linkreader.get_text( target ) ),
                                    template_text, 1 )       
        # parser end
        file_name = file_name.replace( self.input_suffix, self.output_suffix )
        print " * output: %s" % targ_path + file_name
        if not os.path.exists( targ_path ):
            os.mkdir( targ_path )
        out_file = open( targ_path + file_name, 'w+' )
        out_file.write( template_text )
        out_file.close()
    
    def get_template_and_menu( self, path ):
        """returns the closest menu and template"""
        tdone = False
        tfile = "nop"
        tpath = path
        mdone = False
        mfile = "nop"
        mpath = path
        while not tdone:
            for x in self.template_files_list:
                if x[0] == tpath:
                    tdone = True
                    tfile = x[0] + x[1]
            if tpath.count('/') == 1 and tfile == "nop":
                print " ! Warning no template was found."
                tdone = True
            tpath = tpath[: tpath[:-1].rfind('/') ] + '/'
        while not mdone:
            for x in self.menu_files_list:
                if x[0] == mpath:
                    mdone = True
                    mfile = x[0] + x[1]
            if mpath.count('/') == 1 and mfile == "nop":
                print " ! Warning no menu was found."
                mdone = True
            mpath = mpath[: mpath[:-1].rfind('/') ] + '/'
        return [ tfile, mfile ]

    def is_ignored( self, file_name ):
        """Check if this file ends with any of the ignored endings"""
        if filter( lambda x: file_name.endswith( x ),
                   self.input_ignore ):
            return True
        
        else:
            return False

if __name__ == '__main__':
    print "vebzeitl v0.0.3"
    print "Copyright (C) 2007-2009  Patrik Lembke."
    print "This is free software with ABSOLUTELY NO WARRANTY"
    print
    if len(sys.argv) == 2:
        main = Main_Program()
        main.parse_settings( sys.argv[1] )
        main.scan_input()
    else:
        print "no config file specified"
        print "vebzeitl config.conf"
